import {HttpClient, HttpHeaders} from '@angular/common/http';
import {Component, Inject} from '@angular/core';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import {MAT_DIALOG_DATA, MatDialogRef} from '@angular/material/dialog';
import { CreateClientComponent } from '../create-client/create-client.component';
import {DataService} from "../../data.service";
import {ToastrService} from "ngx-toastr";

@Component({
  selector: 'app-edit-client',
  templateUrl: './edit-client.component.html',
  styleUrl: './edit-client.component.css'
})
export class EditClientComponent {
  clientForm!: FormGroup;
  parentUnits: any[] = []; // Array to store parent units fetched from API
  hardwareProfiles: any[] = []; // Array to store hardware profiles fetched from API
  isEditMode: boolean; // Flag to check if it's edit mode
  protected netifaceTypes = [
    {"name": 'Eth0', "value": "eth0"},
    {"name": 'Eth1', "value": "eth1"},
    {"name": 'Eth2', "value": "eth2"},
  ];
  protected netDriverTypes = [
    {"name": 'Generic', "value": "generic"},
  ];
  loading:boolean = false

  constructor(
    private fb: FormBuilder,
    private dialogRef: MatDialogRef<CreateClientComponent>,
    private http: HttpClient,
    private dataService: DataService,
    private toastService: ToastrService,
    @Inject(MAT_DIALOG_DATA) public data: any // Inject data for edit mode
  ) {
    this.isEditMode = !!data?.uuid; // Check if uuid is passed to determine edit mode
    if (this.isEditMode) {
      this.loadData(data.uuid);
    }
  }

  ngOnInit(): void {
    this.loadParentUnits(); // Load parent units when component initializes
    this.loadHardwareProfiles(); // Load hardware profiles when component initializes
    this.clientForm = this.fb.group({
      organizationalUnit: [null,Validators.required],
      name: ['', Validators.required],
      serialNumber: null,
      netiface: null,
      netDriver: null,
      mac: null,
      ip: null,
      menu: null,
      hardwareProfile: null,
    });
  }

  loadParentUnits() {
    const url = 'http://127.0.0.1:8080/organizational-units?page=1&itemsPerPage=10000';

    this.http.get<any>(url).subscribe(
      response => {
        this.parentUnits = response['hydra:member'];
      },
      error => {
        console.error('Error fetching parent units:', error);
      }
    );
  }

  loadHardwareProfiles(): void {
    this.dataService.getHardwareProfiles().subscribe(
      (data: any[]) => {
        this.hardwareProfiles = data;
      },
      (error: any) => {
        console.error('Error fetching hardware profiles', error);
      }
    );
  }

  loadData(uuid: string) {
    this.loading = true
    const url = `http://127.0.0.1:8080/clients/${uuid}`;

    this.http.get<any>(url).subscribe(
      data => {
        this.clientForm.patchValue({
          name: data.name,
          ip: data.ip,
          mac: data.mac,
          netiface: data.netiface,
          netDriver: data.netDriver,
          serialNumber: data.serialNumber,
          hardwareProfile: data.hardwareProfile ? data.hardwareProfile['@id'] : null,
          organizationalUnit: data.organizationalUnit ? data.organizationalUnit['@id'] : null
        });
        this.loading = false
      }
    );
  }

  onSubmit() {
    console.log('Form data:', this.clientForm.value);
    if (this.clientForm.valid) {
      const formData = this.clientForm.value;

      if (this.isEditMode) {
        // Edit mode: Send PUT request to update the unit
        const putUrl = `http://127.0.0.1:8080/clients/${this.data.uuid}`;
        const headers = new HttpHeaders({ 'Content-Type': 'application/json' });

        this.http.patch<any>(putUrl, formData, { headers }).subscribe(
          response => {
            console.log('PUT successful:', response);
            this.dialogRef.close();
            this.openSnackBar(false, 'Cliente actualizado exitosamente');

          },
          error => {
            console.error('Error al realizar PUT:', error);
            this.openSnackBar(true, 'Error al actualizar el cliente: ' + error.error['hydra:description']);
          }
        );
      } else {
        // Create mode: Send POST request to create a new unit
        const postUrl = 'http://127.0.0.1:8080/clients';
        const headers = new HttpHeaders({ 'Content-Type': 'application/json' });

        this.http.post<any>(postUrl, formData, { headers }).subscribe(
          response => {
            console.log('POST successful:', response);
            this.dialogRef.close();
            this.openSnackBar(false, 'Cliente creado exitosamente');
          },
          error => {
            console.error('Error al realizar POST:', error);
            this.openSnackBar(true, 'Error al crear el cliente: ' + error.error['hydra:description']);
          }
        );
      }
    }
  }

  onNoClick(): void {
    this.dialogRef.close();
  }

  openSnackBar(isError: boolean, message: string) {
    if (isError) {
      this.toastService.error(' Error al crear el cliente: ' + message, 'Error');
    } else
      this.toastService.success(message, 'Éxito');
  }
}
